#!/usr/bin/env python
#----------------------------------------------------------------------------
# Name:         PyTAS.py
# Purpose:      A Python based version of ISU TAS
#
# Author:       Matthew Patitz
#
# Created:      
# RCS-ID:       
# Copyright:    
# License:      
#----------------------------------------------------------------------------

"""
This simulator is designed to simulate aTAM systems based on the ISU TAS format

Requirements:
    This application requires Python 2.x due to its dependency on wxPython

Dependencies:
    wxPython    (download installer for your platform here: https://wxpython.org/Phoenix/snapshot-builds/
                I used this command:  "pip install -U --pre -f https://wxpython.org/Phoenix/snapshot-builds/ wxPython"
                for Windows and Mac (sudo -H pip install wxPython), and additional instructions can be found here:
                https://wxpython.org/Phoenix/snapshot-builds/README.txt)
    pil         (via pillow, install using 'pip install pillow', pip can be found in your Python27\Scripts dir)
    svgwrite    (via 'pip install svgwrite')

TODO:
    - implement tile type window to more clearly show tile type labels
    - create cached set of bitmaps for tile types (with and without labels)
    - implement clickable assembly overview window
    - interface for TAScript
    - tile type editor
    - packages for Windows and OS X

"""

#
#Previous dependencies no longer used:
#    pyPdf2      (install with 'pip install pypdf2')
#    reportlab   (install with 'pip install reportlab')


#import wxversion
#wxversion.select('3.0-msw', optionsRequired=True)
#wxversion.select('3.0-Phoenix', optionsRequired=True)
import wx
import wx.richtext as rt
from OctTASAssembly import *
from OctTASCanvas import *
from version import OctTASVersion
from SimulatorFramework.version import SimAppVersion
from SimulatorFramework import SimApp
#from TileType import *
import wx.lib.agw.cubecolourdialog as CCD
#from wx.lib.pubsub import Publisher

#===============================================================================
# from reportlab.lib.pagesizes import letter
# from reportlab.lib.units import inch
# from reportlab.pdfgen import canvas
# from reportlab.platypus import SimpleDocTemplate, Image
#===============================================================================

#---------------------------------------------------------------------------


class OctTASApp(SimApp):
    def __init__(self):
        SimApp.__init__(self)

    def OnInit(self):
        return SimApp.OnInit(self, title="OctTAS Simulator")

    def CreateCanvas(self):
        self.canvas = MyCanvas(self.frame, wx.ID_ANY, self.frame, wx.DefaultSize, self)
        
    def GetVersionString(self):
        return OctTASVersion + " (SimApp version " + SimAppVersion + ")"
        
    def OnAbout(self, event):
        versionString = self.GetVersionString()
        aboutInfo = wx.adv.AboutDialogInfo()
        aboutInfo.SetName("OctTAS")
        aboutInfo.SetVersion(versionString)
        aboutInfo.SetDescription("Octagonal-tile self-assembly simulator")
        #aboutInfo.SetCopyright("(C) 2017-2019")
        aboutInfo.SetWebSite("http://www.self-assembly.net")
        aboutInfo.AddDeveloper("Matthew Patitz")
        wx.adv.AboutBox(aboutInfo)


#----------------------------------------------------------------------------

app = OctTASApp()
app.MainLoop()

#---------------------------------------------------------------------------
