from SimulatorFramework import SimUtils,TileType,Tile,TileAssembly
from SimulatorFramework import Frontier
from OctTASTileType import OctTASTileType,DirNameArray
import os
import sys
import random


#===============================================================================
# N  = 0
# NE = 1
# E  = 2
# SE = 3
# S  = 4
# SW = 5
# W  = 6
# NW = 7
# ALLDIRS = 8
# DIRSTRS = ("N", "NE", "E", "SE", "S", "SW", "W", "NW")
#===============================================================================
NBRS = [(0,1),(1,1),(1,0),(1,-1),(0,-1),(-1,-1),(-1,0),(-1,1)]
#NBRS = [(0,1),(1,0),(0,-1),(-1,0),]

#----------------------------------------------------------------------------
        
class OctTASAssembly(TileAssembly.Assembly):

    def __init__(self, tileWidths=100, tileHeights=100):
        TileAssembly.Assembly.__init__(self, tileWidths, tileHeights)

        self.temperature = 0
        self.FrontierHashTable = dict()
        self.imgDict = dict()
        self.frontierHashTableHits = 0
        self.frontierHashTableMisses = 0

    def ClearSpace(self):
        TileAssembly.Assembly.ClearSpace(self)
        self.FrontierHashTable = dict()
        self.frontierHashTableHits = 0
        self.frontierHashTableMisses = 0

    def GetFittingTilesForLoc(self, coords):
        retSet = None
        count = 0
        nbrGlues = []
        nbrList = []
        incidentStrengthTot = 0
        for n in NBRS:
            nbrCoords = (coords[0]+n[0], coords[1]+n[1])
            nbrTile = self.GetLoc(nbrCoords, True)
            if not nbrTile is None:
                nbrList.append(count)
                oppDir = (count + 4) % 8
                nbrGlues.append(nbrTile.type.sides[oppDir].Hash())
                incidentStrengthTot = incidentStrengthTot + nbrTile.type.sides[oppDir].strength
                #if retSet is None:
                #    retSet = nbrTile.type.matches[oppDir]
                #else:
                #    retSet = retSet.intersection(nbrTile.type.matches[oppDir])
            else:
                nbrGlues.append(None)
            count = count + 1

        # If there aren't strong enough incident glues to allow a tile to bind, get outta here
        if incidentStrengthTot < self.temperature:
            return None

        signature = str(nbrGlues)
        #print("At coords " + str(coords) + " signature is " + signature)
        if signature in self.FrontierHashTable:
            self.frontierHashTableHits = self.frontierHashTableHits + 1
            #print("cache hits: " + str(self.frontierHashTableHits))
            return self.FrontierHashTable[signature]
        else:
            self.frontierHashTableMisses = self.frontierHashTableMisses + 1
            #print("cache misses: " + str(self.frontierHashTableMisses))

        for t in self.tileTypes:
            if not "Seed" in t and not "SEED" in t:
                totStr = 0
                #for n in range(len(NBRS)):
                for n in nbrList:
                    if not nbrGlues[n] is None:
                        if nbrGlues[n] == self.tileTypes[t].sides[n].Hash():
                            totStr = totStr + self.tileTypes[t].sides[n].strength
                #print("totStr: " + str(totStr))
                            if totStr >= self.temperature:
                                if retSet is None:
                                    retSet = set()
                                retSet.add(t)
                                break
        self.FrontierHashTable[signature] = retSet

        if not retSet is None and len(retSet) > 1:
            hasGrowth    = False
            hasNonGrowth = False
            bHasBadNondet = False
            for tt in retSet:
                if tt.startswith("growth"):
                    hasGrowth = True
                else:
                    hasNonGrowth = True
            if hasGrowth == True:
                SimUtils.globalApp.OutputMessage("Nondeterminism in growth frontier location " + str(coords) + " with " + str(len(retSet)) + " choices: " + str(retSet))
                if hasNonGrowth == True:
                    SimUtils.globalApp.OutputMessage("Mixture of growth and non-growth tiles can fit in frontier location " + str(coords))
                    bHasBadNondet = True
            if hasGrowth == False or hasNonGrowth == True:
                prefix = min(retSet).split(" ")[0]
                if "-" in prefix:
                    prefix = prefix.split("-")[0]
                for tt in retSet:
                    prefix2 = tt.split(" ")[0]
                    if "-" in prefix2:
                        prefix2 = prefix2.split("-")[0]
                    if not prefix == prefix2:
                        SimUtils.globalApp.OutputMessage("Different tile types modulo rotation (" + prefix + ", " + prefix2 + ") fit in frontier location " + str(coords))
                        bHasBadNondet = True
            if bHasBadNondet == True:
                outputSidesSet = []
                for tt in retSet:
                    outputSides = []
                    d = 0
                    for n in nbrGlues:
                        if n is None:
                            outputSides.append(self.tileTypes[tt].sides[d].label)
                        else:
                            outputSides.append(None)
                        d = d + 1
                    outputSidesSet.append(outputSides)
                SimUtils.globalApp.OutputMessage("  Exposed output sides:")
                bHasBadNondet = False
                os0 = outputSidesSet[0]
                for os in outputSidesSet:
                    if not os0 == os:
                        bHasBadNondet = True
                for os in outputSidesSet:
                    SimUtils.globalApp.OutputMessage("    " + str(os))
                if bHasBadNondet == True:
                    SimUtils.globalApp.canvas.keepGoing = False
            #SimUtils.globalApp.OutputMessage("Nondeterminism in frontier location " + str(coords) + " with " + str(len(retSet)) + " choices: " + str(retSet))
        return retSet
        
    def LoadTileSystem(self, fileName, backgroundColor=None):
        if len(fileName) < 1:
            return False
        elif os.path.isabs(fileName) == False:
            fileName = "./" + fileName
            fileName = os.path.abspath(fileName)

        #try:
        os.stat(fileName)
        rFile = open(fileName, "r")
        SimUtils.globalApp.OutputMessage("Using tile system definition file '" + fileName + "'")
        lines = rFile.readlines()
        rFile.close()
        
        self.tileTypes = dict()
        self.seedPatch = []
        self.frontier = set()
        self.temperature = 2    # Default temperature = 2
        
        tdsFileName = lines[0].strip()
        if os.path.isabs(tdsFileName) == False:
            tdsFileName = os.path.dirname(fileName) + "/" + tdsFileName
            tdsFileName = os.path.abspath(tdsFileName)
        directory = os.path.dirname(tdsFileName)
        self.LoadTileSet(tdsFileName, backgroundColor=backgroundColor, directory=directory)
        
        for line in lines[1:]:
            if line.startswith("Temperature="):
                temperatureStr = line[len("Temperature="):]
                self.temperature = int(temperatureStr)
            else:
                seedLocArr = line.split(" ")
                seedTileName = seedLocArr[0]
                seedTileXLoc = int(seedLocArr[1])
                seedTileYLoc = int(seedLocArr[2])
                seedTileZLoc = 0
                if len(seedLocArr) > 3:
                    seedTileZLoc = int(seedLocArr[3])
                    if not seedTileZLoc == 0:
                        print("ERROR: The seed contains z-coordinates not equal to 0.  3D mode is not currently supported.")
                self.seedPatch.append((seedTileName, (seedTileXLoc, seedTileYLoc)))

        # Load tile set, then verify that tile types referenced in the seed actually exist
        mid = self.PlaceSeed()
        
        self.tileSystemDef = dict()
        self.tileSystemDef['tdsFile'] = tdsFileName
        self.tileSystemDef['temperature'] = self.temperature
        self.tileSystemDef['directory'] = directory
        
        return mid
        
    def CreateTileTypeImages(self, backgroundColor=None):
        for ttypeName in self.tileTypes:
            tt = self.tileTypes[ttypeName]
            if tt.imgFile is None:
                print("ERROR: No image for tile type " + ttypeName)
            else:
                if self.imgDict.has_key(tt.imgFile) == True:
                    tt.img = self.imgDict[tt.imgFile]
                else:
                    tt.MakeImage(backgroundColor=backgroundColor)
                    self.imgDict[tt.imgFile] = tt.img
        
    def LoadTileSet(self, tdsFileName, backgroundColor=None, directory=None):

        SimUtils.globalApp.OutputMessage("Reading tile set from '" + tdsFileName + "'")
        self.imgDict = dict()
        os.stat(tdsFileName)
        rFile = open(tdsFileName, "r")
        lines = rFile.readlines()
        rFile.close()

        i = 0
        while i < len(lines):
            lineArr = lines[i].strip().split(" ")
            if lineArr[0] == "TILENAME":
                ttype = OctTASTileType()
                ttype.name = lineArr[1]
            elif lineArr[0] == "LABEL" and len(lineArr) > 1:
                ttype.label = lineArr[1]
            elif lineArr[0] == "TILECOLOR" and len(lineArr) > 1:
                ttype.color = lineArr[1]
            elif lineArr[0] == "TILEIMAGE" and len(lineArr) > 1:
                ttype.imgFile = " ".join(lines[i].strip().split()[1:])
                if not directory is None:
                    ttype.imgFile = directory + "/" + ttype.imgFile
            elif lineArr[0] == "CREATE":
                self.tileTypes[ttype.name] = ttype
            else:
                for d in range(len(DirNameArray)):
                    dirName = DirNameArray[d]
                    if lineArr[0] == dirName + "LABEL" and len(lineArr) > 1:
                        ttype.sides[d].label = lineArr[1]
                    elif lineArr[0] == dirName + "BIND" and len(lineArr) > 1:
                        ttype.sides[d].strength = int(lineArr[1])
            i = i + 1

        self.CreateTileTypeImages(backgroundColor=backgroundColor)

        SimUtils.globalApp.OutputMessage("Loaded tile set with size: " + str(len(self.tileTypes)))

        return True

